import time

from pycrm114.storage import Storage


class RedisStorage(Storage):
    def update_data_block(self, data_block):
        self._client.hset("{}:crm".format(self._namespace), 'last_modified', time.time())
        self._client.hset("{}:crm".format(self._namespace), 'data_block', data_block.dumps())

    def __init__(self, client=None, namespace=None):
        self._client = client
        self._namespace = namespace
        self._lock = None

    @property
    def data_block_string(self):

        return self._client.hget("{}:crm".format(self._namespace), 'data_block')

    @property
    def last_updated(self):
        return self._client.hget("{}:crm".format(self._namespace), "last_modified")

    def acquire_lock(self):
        self._lock = self._client.lock("{}:crm:lock".format(self._namespace))
        self._lock.acquire()

    def release_lock(self):
        if self._lock:
            self._lock.release()
            self._lock = None





import pycrm114._binding

CRM114_BOOST = pycrm114._binding.CRM114_BOOST
CRM114_BYCHAR = pycrm114._binding.CRM114_BYCHAR
CRM114_CHI2 = pycrm114._binding.CRM114_CHI2
CRM114_CORRELATE = pycrm114._binding.CRM114_CORRELATE
CRM114_CROSSLINK = pycrm114._binding.CRM114_CROSSLINK
CRM114_DEFAULT = pycrm114._binding.CRM114_DEFAULT
CRM114_ENTROPY = pycrm114._binding.CRM114_ENTROPY
CRM114_ERASE = pycrm114._binding.CRM114_ERASE
CRM114_FSCM = pycrm114._binding.CRM114_FSCM
CRM114_HYPERSPACE = pycrm114._binding.CRM114_HYPERSPACE
CRM114_MARKOVIAN = pycrm114._binding.CRM114_MARKOVIAN
CRM114_MICROGROOM = pycrm114._binding.CRM114_MICROGROOM
CRM114_NEURAL_NET = pycrm114._binding.CRM114_NEURAL_NET
CRM114_OSB = pycrm114._binding.CRM114_OSB
CRM114_OSBF = pycrm114._binding.CRM114_OSBF
CRM114_OSBF_BAYES = pycrm114._binding.CRM114_OSBF_BAYES
CRM114_OSB_BAYES = pycrm114._binding.CRM114_OSB_BAYES
CRM114_OSB_WINNOW = pycrm114._binding.CRM114_OSB_WINNOW
CRM114_PCA = pycrm114._binding.CRM114_PCA
CRM114_READLINE = pycrm114._binding.CRM114_READLINE
CRM114_REFUTE = pycrm114._binding.CRM114_REFUTE
CRM114_STRING = pycrm114._binding.CRM114_STRING
CRM114_SVM = pycrm114._binding.CRM114_SVM
CRM114_UNIGRAM = pycrm114._binding.CRM114_UNIGRAM
CRM114_UNIQUE = pycrm114._binding.CRM114_UNIQUE
CRM114_WINNOW = pycrm114._binding.CRM114_WINNOW



from abc import ABCMeta, abstractproperty, abstractmethod
import six


@six.add_metaclass(ABCMeta)
class Storage(object):
    @abstractproperty
    def control_block_file(self):
        raise NotImplementedError

    @abstractproperty
    def data_block_file(self):
        raise NotImplementedError

    @abstractmethod
    def update_control_block(self, control_block):
        raise NotImplementedError

    @abstractmethod
    def update_data_block(self, data_block):
        raise NotImplementedError

    def save(self, control_block, data_block):
        self.update_control_block(control_block)
        self.update_data_block(data_block)


from . fs import *


from . import flags
from . import _binding
from pycrm114.errors import CRM114InitializationError


class CRM114(object):
    def __init__(self, classes, flags=flags.CRM114_OSB, storage=None, auto_save=False):
        self.classes = classes
        self.flags = flags
        self.storage = storage
        self.auto_save = auto_save
        self.control_block = None
        self.data_block = None
        self.__init_crm()

    def __init_crm(self):
        cb_location = None
        data_location = None
        if self.storage:
            cb_location = self.storage.control_block_file
            data_location = self.storage.data_block_file
        if data_location:
            self.data_block = _binding.DataBlock.load(open(data_location))
        elif cb_location:
            self.control_block = _binding.ControlBlock.load(open(cb_location))
        else:
            try:
                self.control_block = _binding.ControlBlock(flags=self.flags,
                                  classes=[(cls, True) for cls in self.classes],
                                  start_mem = 8000000)
            except _binding.error as e:
                raise CRM114InitializationError("Unable to initialize (%s)" % e)
            self.data_block = _binding.DataBlock(self.control_block)

    def learn(self, cls, text):
        self.data_block.learn_text(self.classes.index(cls), text)
        if self.auto_save and self.storage:
            self.storage.save(self.control_block, self.data_block)

    def forget(self, cls, text):
        self.data_block.forget_text(self.classes.index(cls), text)
        if self.auto_save and self.storage:
            self.storage.save(self.control_block, self.data_block)

    def classify(self, text):
        match_result = self.data_block.classify_text(text)
        best_match = match_result.best_match()
        score = [k['prob'] for k in match_result.scores() if k['name'] == best_match][0]
        return {"class": best_match, "score": score}

    def save(self):
        if self.storage:
            return self.storage.save(self.control_block, self.data_block)



import os
import platform
from setuptools import setup, Extension, find_packages


top_dir = os.path.dirname(os.path.realpath(__file__))
inc_dir = os.path.join(top_dir, "include")
lib_dir = os.path.join(top_dir, "lib")
libcrm114_obj = [
    os.path.join(top_dir, "lib", f) for f in (
        "crm114_base.c",
        "crm114_markov.c",
        "crm114_markov_microgroom.c",
        "crm114_bit_entropy.c",
        "crm114_hyperspace.c",
        "crm114_svm.c",
        "crm114_svm_lib_fncts.c",
        "crm114_svm_quad_prog.c",
        "crm114_fast_substring_compression.c",
        "crm114_pca.c",
        "crm114_pca_lib_fncts.c",
        "crm114_matrix.c",
        "crm114_matrix_util.c",
        "crm114_datalib.c",
        "crm114_vector_tokenize.c",
        "crm114_strnhash.c",
        "crm114_util.c",
        "crm114_regex_tre.c",
    )
]
if platform.uname()[0] in ['Darwin', 'BSD']:
    libcrm114_obj.insert(0, 'lib/memstream.c')
    libcrm114_obj.insert(0, 'lib/fmemopen.c')

pycrm114_module = Extension('pycrm114._binding',
                            sources=libcrm114_obj + ['pycrm114/pycrm114_module.c'],
                            include_dirs=[inc_dir],
                            library_dirs=[lib_dir],
                            runtime_library_dirs=[lib_dir],
                            libraries=['tre'],
                            extra_compile_args=['-std=c99', '-g', '-pedantic', '-Wall', '-Wextra', '-Wpointer-arith',
                                                '-Wstrict-prototypes', '-fpic'])

requirements = filter(None, open(
    os.path.join(top_dir, 'requirements', 'main.txt')).read().splitlines())

import versioneer

versioneer.versionfile_source = "pycrm114/_version.py"
versioneer.versionfile_build = "pycrm114/version.py"
versioneer.tag_prefix = ""
versioneer.parentdir_prefix = "pycrm114-"


setup(
    name='pycrm114',
    version=versioneer.get_version(),
    description='Python interface to libcrm114',
    long_description=open('README.rst').read() + open('HISTORY.rst').read(),
    author='Prashanth Mundkur',
    author_email='prashanth.mundkur at gmail.com',
    maintainer='Ali-Akber Saifee',
    maintainer_email='ali at indydevs.org',
    url='https://github.com/alisaifee/pycrm114',
    license=open('LICENSE').read(),
    install_requires=requirements,
    ext_modules=[pycrm114_module],
    packages=find_packages(exclude=["tests*"]),
    cmdclass=versioneer.get_cmdclass(),
)



import os
from pycrm114.storage import Storage


class FileSystemStorage(Storage):
    def update_control_block(self, control_block):
        control_block.dump(open(self._controlblock_file, "w"))

    def update_data_block(self, data_block):
        data_block.dump(open(self._datablock_file, "w"))

    def __init__(self, data_dir):
        if not os.path.isdir(data_dir):
            os.makedirs(data_dir)
        self.data_dir = data_dir
        self._datablock_file = os.path.join(self.data_dir, "datablock.txt")
        self._controlblock_file = os.path.join(self.data_dir, "controlblock.txt")
    @property
    def data_block_file(self):
        return self._datablock_file if os.path.isfile(self._datablock_file) else None

    @property
    def control_block_file(self):
        return self._controlblock_file if os.path.isfile(self._controlblock_file) else None
